unit ColorPickers;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TColorChangeEvent = procedure (Sender: TObject; NewColor: TColor;
    var CanChange: Boolean) of object;

  TColorPick = class(TCustomControl)
  private
    FColor: TColor;
    FOnCanChange: TColorChangeEvent;
    FOnChange: TNotifyEvent;
    procedure PopupClick(Sender: TObject);
  protected
    function CanChange(NewColor: TColor): Boolean; virtual;
    procedure Changed; virtual;
    procedure Paint; override;
    procedure SetColor(Value: TColor); virtual;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align;
    property Anchors;
    property Color: TColor read FColor write SetColor default clBlue;
    property Constraints;
    property DockSite;
    property DragCursor;
    property DragKind;
    property DragMode;
    property ParentShowHint;
    property ShowHint;
    property Visible;
    property OnCanChange: TColorChangeEvent read FOnCanChange write FOnCanChange;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDockDrop;
    property OnDockOver;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
    property OnUnDock;
  end;

procedure Register;

implementation

uses Menus;

procedure Register;
begin
  RegisterComponents('Samples', [TColorPick]);
end;

{ TColorPick }

constructor TColorPick.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 100;
  Height := 100;
  FColor := clBlue;
  PopupMenu := NewPopupMenu(Self, '', paLeft, True,
    [NewItem('Change Color', 0, False, True, PopupClick, 0, '')]);
end;

function TColorPick.CanChange(NewColor: TColor): Boolean;
begin
  Result := True;
  if Assigned(FOnCanChange) then FOnCanChange(Self, NewColor, Result);
end;

procedure TColorPick.Changed;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TColorPick.Paint;
begin
  Canvas.Brush.Color := FColor;
  Canvas.Rectangle(0, 0, Width, Height);
end;

procedure TColorPick.SetColor(Value: TColor);
begin
  if (FColor <> Value) and CanChange(Value) then
  begin
    FColor := Value;
    Invalidate;
    Changed;
  end;
end;

procedure TColorPick.PopupClick(Sender: TObject);
begin
  with TColorDialog.Create(Self) do
  begin
    if Execute then Self.Color := Color;
    Free;
  end;
end;

end.
 